from funcy_chain import chain


def test_nlargest():
    assert chain([]).nlargest(0).value == []
    assert chain([]).nlargest(1).value == []
    assert chain([1, 3, 2, 4, 3]).nlargest(2).value == [4, 3]
    assert chain([1, 3, 2, 4, 3]).nlargest(3).value == [4, 3, 3]
    assert chain([1, 3, 2, 4, 1]).nlargest(0).value == []


def test_nsmallest():
    assert chain([]).nsmallest(0).value == []
    assert chain([]).nsmallest(1).value == []
    assert chain([1, 3, 2, 4, 1]).nsmallest(2).value == [1, 1]
    assert chain([1, 3, 2, 4, 1]).nsmallest(3).value == [1, 1, 2]
    assert chain([1, 3, 2, 4, 1]).nsmallest(0).value == []



import pytest

from funcy_chain import chain


def test_enumerate():
    assert chain([]).enumerate().value == []
    assert chain([1, 2, 3]).enumerate().value == [(0, 1), (1, 2), (2, 3)]


def test_max():
    assert chain([1, 3, 2]).max().value == 3
    assert chain([1, 3, 2]).max(key=lambda x: -x).value == 1

    with pytest.raises(ValueError):
        chain([]).max()


def test_min():
    assert chain([1, 3, 2]).min().value == 1
    assert chain([1, 3, 2]).min(key=lambda x: -x).value == 3

    with pytest.raises(ValueError):
        chain([]).min()


def test_sort():
    assert chain([]).sort().value == []
    assert chain([1, 3, 2]).sort().value == [1, 2, 3]
    assert chain([1, 3, 2]).sort(reverse=True).value == [3, 2, 1]
    assert chain([1, 3, 2]).sort(key=lambda x: -x).value == [3, 2, 1]
    assert chain([1, 3, 2]).sort(key=lambda x: -x, reverse=True).value == [1, 2, 3]
    assert chain([1, 3, 2]).sort(lambda x: -x, True).value == [1, 2, 3]


def test_reduce():
    add = lambda a, b: a + b

    # empty lists
    assert chain([]).reduce(add, 0).value == 0
    assert chain([]).reduce(add, []).value == []
    with pytest.raises(Exception):
        chain([]).reduce(add).value

    # adding ints
    assert chain([1, 2, 3]).reduce(add).value == 6
    assert chain([1, 2, 3]).reduce(add, 0).value == 6
    assert chain([1, 2, 3]).reduce(add, 4).value == 10

    # list concatenation
    assert chain([[1], [2]]).reduce(add).value == [1, 2]
    assert chain([[1], [2]]).reduce(add, []).value == [1, 2]
    with pytest.raises(TypeError):
        chain([[1], [2]]).reduce(add, 0).value


def test_reverse():
    assert chain([]).reverse().value == []
    assert chain([1, 2, 3]).reverse().value == [3, 2, 1]
    assert chain([1, 2, 3]).reverse().reverse().value == [1, 2, 3]
    assert chain([[1], [2]]).reverse().value == [[2], [1]]



from funcy_chain import chain


def test_names():
    data = {
        "user1": {
            "firstname": "Alice",
            "lastname": "Liddle",
        },
        "user2": {
            "firstname": "Bob",
            "lastname": "Kennedy",
        }
    }
    names = (
        chain(data)
            .values()
            .map(lambda user_data: "{lastname}, {firstname}".format(**user_data))
            .sort()
    ).value
    assert names == ["Kennedy, Bob", "Liddle, Alice"]


def test_integers():
    assert (
        chain([1, 2, 3, 7, 6, 5, 4])
            .without(3)
            .filter(lambda x: x > 2)
            .remove(lambda x: x > 6)
            .sort(reverse=True)
    ).value == [6, 5, 4]



import pytest

from funcy_chain import chain


def test_items():
    assert chain({}).items().value == []
    assert chain({1: 2}).items().value == [(1, 2)]

    with pytest.raises((AttributeError, TypeError)):
        assert chain([]).items().value


def test_keys():
    assert chain({}).keys().value == []
    assert chain({1: 2}).keys().value == [1]

    with pytest.raises((AttributeError, TypeError)):
        assert chain([]).keys().value


def test_values():
    assert chain({}).values().value == []
    assert chain({1: 2}).values().value == [2]

    with pytest.raises((AttributeError, TypeError)):
        assert chain([]).values().value


def test_update():
    assert chain({}).update().value == {}
    assert chain({}).update({"b": 2}).value == {"b": 2}
    assert chain({}).update([("b", 2)]).value == {"b": 2}
    assert chain({}).update(b=2).value == {"b": 2}
    assert chain({"a": 1}).update({"b": 2}).value == {"a": 1, "b": 2}
    assert chain({"a": 1}).update({"a": 2}).value == {"a": 2}

    with pytest.raises((AttributeError, TypeError)):
        assert chain([]).update().value



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['funcy_chain']

package_data = \
{'': ['*']}

install_requires = \
['funcy']

extras_require = \
{'dev': ['flit >=3.3, <4'],
 'lint': ['black', 'isort', 'pylint'],
 'test': ['pytest >=2.7.3', 'tox >=3, <4']}

setup(name='funcy-chain',
      version='0.1.1',
      description='Method chaining with funcy.',
      author=None,
      author_email='Tal Einat <taleinat@gmail.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.5',
     )
